/*********************************************************************
 * Capítulo 3. Tutorial de desarrollo con OpenFL. Gestión de capas.
 * Programación Multimedia y Juegos
 * Autor: David Vallejo Fernández    david.vallejo@openflbook.com
 * Autor: Carlos González Morcillo   carlos.gonzalez@openflbook.com
 * Autor: David Frutos Talavera      david.frutos@openflbook.com
 *
 * You can redistribute and/or modify this file under the terms of the
 * GNU General Public License ad published by the Free Software
 * Foundation, either version 3 of the License, or (at your option)
 * and later version. See <http://www.gnu.org/licenses/>.
 *
 * This file is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.  
 *********************************************************************/

package book.openfl.ordenCapas;

import aze.display.TileLayer;
import aze.display.TileSprite;
import aze.display.SparrowTilesheet;
import openfl.Assets;
import flash.display.Sprite;
import flash.events.Event;
import flash.ui.Keyboard;
import flash.events.KeyboardEvent;
import flash.Lib;
import flash.net.URLLoader;
import flash.net.URLRequest;

class LayerExample extends Sprite{

  var _layerBg:TileLayer;           // Capa del Background (Bg)
  var _layerFg:TileLayer;           // Capa del Foreground (Fg)
  // var _hashLayer:Hash<TileSprite>;  // Hash de los sprites capa Fg
  // esto cambia a map
  var _hashLayer:Map<String,TileSprite>;
  var sheetData:String; 
  // Constructor =====================================================

	private function new() {
    super();
			#if js
		// workaround for binary data loading bug in html5
		var ul = new URLLoader(new URLRequest("img/capas.xml")); 
		ul.addEventListener(Event.COMPLETE, ul_complete);
		#else
		sheetData = Assets.getText("img/capas.xml");
		createScene();
		#end
	}
	 function createScene():Void {
		 
		var tilesheet:SparrowTilesheet = new SparrowTilesheet
		  (Assets.getBitmapData("img/capas.png"), sheetData);
		_layerBg = new TileLayer(tilesheet);  
		_layerFg = new TileLayer(tilesheet);
		_hashLayer = new Map<String, TileSprite>();

		addSprite(_layerBg, "Fondo", 100, 100);
		addChild(_layerBg.view);

		addSprite (_layerFg, "CapaA", 150, 150, _hashLayer);
		addSprite (_layerFg, "CapaS", 200, 200, _hashLayer);
		addSprite (_layerFg, "CapaD", 250, 250, _hashLayer);
		addChild(_layerFg.view);

		Lib.current.stage.addEventListener(KeyboardEvent.KEY_DOWN, onKeyPressed);
		Lib.current.stage.addEventListener(Event.ENTER_FRAME, onEnterFrame);
	}  	
	function ul_complete(e)
	{
		sheetData = e.target.data;
		createScene();
	}

  // addSprite =======================================================
  function addSprite(layer:TileLayer, sprname:String, x:Int, y:Int,  
		     map:Map<String, TileSprite>=null):Void {
    var spr = new TileSprite(layer, sprname);
    spr.x = x; spr.y = y; layer.addChild(spr);
    //if (hash != null) hash.set(sprname, spr);  // Aniadimos a Hash
	// esto pasa a map
	if (map != null) map.set(sprname, spr);
  }

  // Update ==========================================================
  function onEnterFrame(event:Event):Void {
    _layerBg.render();  
    for (e in _hashLayer) e.x += 1-Std.random(3);
    _layerFg.render();
  }
  
  // Events ==========================================================
  function onKeyPressed(event:KeyboardEvent):Void {
    var spr : TileSprite = null;
    var keyString : String = null; 
    switch (event.keyCode) {
    case Keyboard.A: 
      keyString = "A";
    case Keyboard.S:
      keyString = "S";
    case Keyboard.D:
      keyString = "D";
    }
	if(keyString!=null){
		spr = _hashLayer.get("Capa" + keyString);
		trace ("Posicion de " + keyString + " : " + _layerFg.indexOf(spr));
		_layerFg.removeChild(spr);
		_layerFg.addChildAt(spr, _layerFg.numChildren);
	}
  }

  // Main ============================================================
  public function main() {
    Lib.current.addChild(new LayerExample());
  }
}
